﻿Imports System.ComponentModel
Imports RecipeEditorViaMC3eJet.MelsecCommunication

Public Class frmDeviceOperation
	Private Enum CommandOffset
		HeaterActive = 0
		HeaterSleep = 1
		N2ValveOpen = 2
		N2ValveClose = 3
		BlowerRun = 4
		BlowerStop = 5
	End Enum

	Private Enum StatusOffset
		HeaterActive = 0
		HeaterSleep = 1
		N2ValveOpen = 2
		N2ValveClose = 3
		BlowerRun = 4
		BlowerStop = 5
	End Enum

	Private Const CommandAddressFrom As Integer = 20
	Private Const StatusReadAddressFrom As Integer = 30
	Private Const PollingInterval As Integer = 200

	Private _QCPU As MelsecCPU

	Private WithEvents BitDeviceWatchWorker As BackgroundWorker


	Public Sub New(CPU As MelsecCPU)
		InitializeComponent()
		AddHandler cmdHeaterActive.Click, AddressOf DeviceOperation_Click
		AddHandler cmdHeaterSleep.Click, AddressOf DeviceOperation_Click
		AddHandler cmdN2ValveOpen.Click, AddressOf DeviceOperation_Click
		AddHandler cmdN2ValveClose.Click, AddressOf DeviceOperation_Click
		AddHandler cmdBlowerRun.Click, AddressOf DeviceOperation_Click
		AddHandler cmdBlowerStop.Click, AddressOf DeviceOperation_Click

		_QCPU = CPU
	End Sub


	Private Sub frmDeviceOperation_Shown(sender As Object, e As EventArgs) Handles Me.Shown
		TopMost = DirectCast(Owner, frmMain).TopMost

		BitDeviceWatchWorker = New BackgroundWorker With {.WorkerReportsProgress = True, .WorkerSupportsCancellation = True}
		BitDeviceWatchWorker.RunWorkerAsync()
	End Sub

	Private Sub frmDeviceOperation_Closing(sender As Object, e As CancelEventArgs) Handles Me.Closing
		BitDeviceWatchWorker.CancelAsync()
		Do Until BitDeviceWatchWorker Is Nothing
			Application.DoEvents()
		Loop
	End Sub


	Private Sub BitDeviceWatchWorker_DoWork(sender As Object, e As DoWorkEventArgs) Handles BitDeviceWatchWorker.DoWork
		Dim ReadResponse As List(Of Boolean)

		If True Then
			'クラスのメソッドを使う場合
			Using BitReader As New BitReader(_QCPU)
				Do Until BitDeviceWatchWorker.CancellationPending
					Threading.Thread.Sleep(PollingInterval)
					ReadResponse = BitReader.Read(New BitAddress(StatusReadAddressFrom, BitDeviceFlags.M), 6)
					BitDeviceWatchWorker.ReportProgress(0, ReadResponse)
				Loop
			End Using

		Else
			'共有メソッドの QuickRead を使う場合
			Do Until BitDeviceWatchWorker.CancellationPending
				Threading.Thread.Sleep(PollingInterval)

				ReadResponse = New List(Of Boolean)
				With ReadResponse
					.Add(BitReader.QuickRead(_QCPU, New BitAddress(StatusReadAddressFrom + 0, BitDeviceFlags.M)))
					.Add(BitReader.QuickRead(_QCPU, New BitAddress(StatusReadAddressFrom + 1, BitDeviceFlags.M)))
					.Add(BitReader.QuickRead(_QCPU, New BitAddress(StatusReadAddressFrom + 2, BitDeviceFlags.M)))
					.Add(BitReader.QuickRead(_QCPU, New BitAddress(StatusReadAddressFrom + 3, BitDeviceFlags.M)))
					.Add(BitReader.QuickRead(_QCPU, New BitAddress(StatusReadAddressFrom + 4, BitDeviceFlags.M)))
					.Add(BitReader.QuickRead(_QCPU, New BitAddress(StatusReadAddressFrom + 5, BitDeviceFlags.M)))
				End With

				BitDeviceWatchWorker.ReportProgress(0, ReadResponse)
			Loop
		End If
	End Sub

	Private Sub BitDeviceWatchWorker_ProgressChanged(sender As Object, e As ProgressChangedEventArgs) Handles BitDeviceWatchWorker.ProgressChanged
		Dim DeviceStatus As List(Of Boolean) = DirectCast(e.UserState, List(Of Boolean))

		Select Case True
			Case DeviceStatus.Item(StatusOffset.HeaterActive) And Not DeviceStatus.Item(StatusOffset.HeaterSleep) : lblHeaterStatus.Text = "ACTIVE"
			Case Not DeviceStatus.Item(StatusOffset.HeaterActive) And DeviceStatus.Item(StatusOffset.HeaterSleep) : lblHeaterStatus.Text = "SLEEP"
			Case Else : lblHeaterStatus.Text = "Unknown"
		End Select

		Select Case True
			Case DeviceStatus.Item(StatusOffset.N2ValveOpen) And Not DeviceStatus.Item(StatusOffset.N2ValveClose) : lblN2ValveStatus.Text = "OPEN"
			Case Not DeviceStatus.Item(StatusOffset.N2ValveOpen) And DeviceStatus.Item(StatusOffset.N2ValveClose) : lblN2ValveStatus.Text = "CLOSE"
			Case Else : lblN2ValveStatus.Text = "Unknown"
		End Select

		Select Case True
			Case DeviceStatus.Item(StatusOffset.BlowerRun) And Not DeviceStatus.Item(StatusOffset.BlowerStop) : lblBlowerFanStatus.Text = "RUN"
			Case Not DeviceStatus.Item(StatusOffset.BlowerRun) And DeviceStatus.Item(StatusOffset.BlowerStop) : lblBlowerFanStatus.Text = "STOP"
			Case Else : lblBlowerFanStatus.Text = "Unknown"
		End Select
	End Sub

	Private Sub BitDeviceWatchWorker_RunWorkerCompleted(sender As Object, e As RunWorkerCompletedEventArgs) Handles BitDeviceWatchWorker.RunWorkerCompleted
		BitDeviceWatchWorker.Dispose()
		BitDeviceWatchWorker = Nothing
	End Sub


	Private Sub DeviceOperation_Click(sender As Object, e As EventArgs)
		'ホントんところは、TcpClient のベースソケットで通信が競合しないようにインターロックをとるべき

		If True Then
			'クラスメソッドを使う場合
			Using BitWriter As New BitWriter(_QCPU)
				Select Case True
					Case sender Is cmdHeaterActive : BitWriter.Write(New BitAddress(CommandAddressFrom + CommandOffset.HeaterActive, BitDeviceFlags.M), {True})
					Case sender Is cmdHeaterSleep : BitWriter.Write(New BitAddress(CommandAddressFrom + CommandOffset.HeaterSleep, BitDeviceFlags.M), {True})
					Case sender Is cmdN2ValveOpen : BitWriter.Write(New BitAddress(CommandAddressFrom + CommandOffset.N2ValveOpen, BitDeviceFlags.M), {True})
					Case sender Is cmdN2ValveClose : BitWriter.Write(New BitAddress(CommandAddressFrom + CommandOffset.N2ValveClose, BitDeviceFlags.M), {True})
					Case sender Is cmdBlowerRun : BitWriter.Write(New BitAddress(CommandAddressFrom + CommandOffset.BlowerRun, BitDeviceFlags.M), {True})
					Case sender Is cmdBlowerStop : BitWriter.Write(New BitAddress(CommandAddressFrom + CommandOffset.BlowerStop, BitDeviceFlags.M), {True})
				End Select
			End Using

		Else
			'共有メソッドの QuickWrite を使う場合
			Select Case True
				Case sender Is cmdHeaterActive : BitWriter.QuickWrite(_QCPU, New BitAddress(CommandAddressFrom + CommandOffset.HeaterActive, BitDeviceFlags.M), True)
				Case sender Is cmdHeaterSleep : BitWriter.QuickWrite(_QCPU, New BitAddress(CommandAddressFrom + CommandOffset.HeaterSleep, BitDeviceFlags.M), True)
				Case sender Is cmdN2ValveOpen : BitWriter.QuickWrite(_QCPU, New BitAddress(CommandAddressFrom + CommandOffset.N2ValveOpen, BitDeviceFlags.M), True)
				Case sender Is cmdN2ValveClose : BitWriter.QuickWrite(_QCPU, New BitAddress(CommandAddressFrom + CommandOffset.N2ValveClose, BitDeviceFlags.M), True)
				Case sender Is cmdBlowerRun : BitWriter.QuickWrite(_QCPU, New BitAddress(CommandAddressFrom + CommandOffset.BlowerRun, BitDeviceFlags.M), True)
				Case sender Is cmdBlowerStop : BitWriter.QuickWrite(_QCPU, New BitAddress(CommandAddressFrom + CommandOffset.BlowerStop, BitDeviceFlags.M), True)
			End Select
		End If

	End Sub
End Class