﻿Imports System.ComponentModel
Imports System.IO
Imports RecipeEditorViaMC3eJet.MelsecCommunication

Public Class frmMain
	Private Const RecipeBlockSize As Integer = 20
	Private Const RecipeTopAddress As Integer = 1000

	Private _CPUAddress As String = "192.168.100.39"
	Private _ConnectionPort As Integer = 8192
	Private _TextEncoding As System.Text.Encoding = PLCEncoding.ShiftJIS

	Private _QCPU As MelsecCPU

	Private WithEvents DebugMonitor As frmDebugMonitor


	Public Sub New()
		InitializeComponent()
		InitializeComponent2()

		DebugMonitor = New frmDebugMonitor With {.Visible = False}
	End Sub


	Private Sub frmMain_Shown(sender As Object, e As EventArgs) Handles Me.Shown
		'
	End Sub

	Private Sub frmMain_Closing(sender As Object, e As CancelEventArgs) Handles Me.Closing
		InitializeComponent2()

		If DebugMonitor IsNot Nothing Then
			DebugMonitor.Dispose()
		End If
	End Sub

	Private Sub InitializeComponent2()
		RemoveHandler spnRecipeIndex.ValueChanged, AddressOf spnRecipeIndex_ValueChanged

		If _QCPU IsNot Nothing Then
			_QCPU.Disconnect()
			_QCPU.Dispose()
			_QCPU = Nothing
		End If

		spnRecipeIndex.Value = spnRecipeIndex.Minimum
		txtTitle.Text = ""
		spnProcessTime.Value = spnProcessTime.Minimum
		spnTemperature.Value = spnTemperature.Minimum
		spnN2Flow.Value = spnN2Flow.Minimum
		tblContentLayout.Enabled = False

		lblConnectionAddress.Text = "-"
		lblEncoding.Text = "-"
	End Sub


	Private Sub QCPU_ConnectionCreated(sender As Object, e As ConnectionCreatedEventArgs)
		RemoveHandler _QCPU.ConnectionCreated, AddressOf QCPU_ConnectionCreated
		RemoveHandler _QCPU.ConnectionFailed, AddressOf QCPU_ConnectionFailed

		lblConnectionAddress.Text = _QCPU.CommClient.Client.RemoteEndPoint.ToString
		lblEncoding.Text = _TextEncoding.ToString

		cmdLoad_Click(sender, e)
		tblContentLayout.Enabled = True

		AddHandler spnRecipeIndex.ValueChanged, AddressOf spnRecipeIndex_ValueChanged
	End Sub

	Private Sub QCPU_ConnectionFailed(sender As Object, e As EventArgs)
		RemoveHandler _QCPU.ConnectionCreated, AddressOf QCPU_ConnectionCreated
		RemoveHandler _QCPU.ConnectionFailed, AddressOf QCPU_ConnectionFailed

		MessageBox.Show("Connection failed.")
		InitializeComponent2()
	End Sub


	Private Sub cmdLoad_Click(sender As Object, e As EventArgs) Handles cmdLoad.Click
		Dim StartAddress As Integer = CInt(RecipeTopAddress + RecipeBlockSize * (spnRecipeIndex.Value - 1))
		Dim PLCResponse As DataBuilder

		Using RecipeReader As New WordReader(_QCPU)
			PLCResponse = New DataBuilder(RecipeReader.Read(New WordAddress(StartAddress, WordDeviceFlags.D), RecipeBlockSize)) With {.DefaultEncoding = _TextEncoding}
		End Using

		txtTitle.Text = PLCResponse.GetString(10)
		spnProcessTime.SetSafety(PLCResponse.GetInt16)
		spnTemperature.SetSafety(CDec(PLCResponse.GetInt16 / 10))
		spnN2Flow.SetSafety(CDec(PLCResponse.GetInt16 / 10))


		'次のようにも書ける
#If DEBUG Then
		Exit Sub

		Dim StartAddress2 As Integer = CInt(RecipeTopAddress + RecipeBlockSize * (spnRecipeIndex.Value - 1))
		Dim PLCResponse2 As New DataBuilder With {.DefaultEncoding = _TextEncoding}

		Using RecipeReader As New WordReader(_QCPU)
			PLCResponse2.SetAnyBytes(RecipeReader.Read(New WordAddress(StartAddress, WordDeviceFlags.D), RecipeBlockSize))
		End Using

		txtTitle.Text = PLCResponse2.GetString(10)
		spnProcessTime.SetSafety(PLCResponse2.GetInt16)
		spnTemperature.SetSafety(CDec(PLCResponse2.GetInt16 / 10))
		spnN2Flow.SetSafety(CDec(PLCResponse2.GetInt16 / 10))
#End If

		'SetSafety
		'NumericUpDown の継承クラス、NumericUpDownEx で追加したコマンド
		'引数が Minimum と Maximum の範囲内かどうか判定するようになってるだけ (^_^;)
	End Sub

	Private Sub cmdUpdate_Click(sender As Object, e As EventArgs) Handles cmdUpdate.Click
		Dim StartAddress As Integer = CInt(RecipeTopAddress + RecipeBlockSize * (spnRecipeIndex.Value - 1))
		Dim WriteData As New DataBuilder With {.DefaultEncoding = _TextEncoding}

		With WriteData
			.SetString(txtTitle.Text, 10)
			.SetInt16(spnProcessTime.Value)
			.SetInt16(spnTemperature.Value * 10)
			.SetInt16(spnN2Flow.Value * 10)
		End With

		Using RecipeWriter As New WordWriter(_QCPU)
			RecipeWriter.Write(New WordAddress(StartAddress, WordDeviceFlags.D), WriteData)
		End Using


		'次のようにも書ける
#If DEBUG Then
		Exit Sub

		Dim StartAddress2 As Integer = CInt(RecipeTopAddress + RecipeBlockSize * (spnRecipeIndex.Value - 1))
		Dim WriteData2 As New List(Of Byte)

		With WriteData2
			.AddRange(DeviceConverter.StringToByteArray(txtTitle.Text, 10, _TextEncoding))
			.AddRange(DeviceConverter.Int16ToByteArray(spnProcessTime.Value))
			.AddRange(DeviceConverter.Int16ToByteArray(spnTemperature.Value))
			.AddRange(DeviceConverter.Int16ToByteArray(spnN2Flow.Value))
		End With

		Using RecipeWriter As New WordWriter(_QCPU)
			RecipeWriter.Write(New WordAddress(StartAddress2, WordDeviceFlags.D), New DataBuilder(WriteData2.ToArray))
		End Using
#End If
	End Sub


	Private Sub mnuFile_DropDownOpening(sender As Object, e As EventArgs) Handles mnuFile.DropDownOpening
		mnuLoad.Enabled = (_QCPU IsNot Nothing)
		mnuSave.Enabled = (_QCPU IsNot Nothing)
		mnuTopMost.Checked = TopMost
	End Sub

	Private Sub mnuLoad_Click(sender As Object, e As EventArgs) Handles mnuLoad.Click
		Using LoadDialog As New OpenFileDialog
			With LoadDialog
				.AddExtension = True
				.AutoUpgradeEnabled = True
				.CheckFileExists = True
				.CheckPathExists = True
				.DefaultExt = "rcj"
				.Filter = "RecipeJet|*.rcj"
				.FilterIndex = 1
				.Multiselect = False
				.ReadOnlyChecked = False
				.ShowHelp = False
				.ShowReadOnly = False
			End With

			If LoadDialog.ShowDialog = DialogResult.OK Then
				Using FileLoader As New StreamReader(LoadDialog.FileName)
					txtTitle.Text = FileLoader.ReadLine
					spnProcessTime.Value = CDec(FileLoader.ReadLine)
					spnTemperature.Value = CDec(FileLoader.ReadLine)
					spnN2Flow.Value = CDec(FileLoader.ReadLine)
				End Using
			End If
		End Using
	End Sub

	Private Sub mnuSave_Click(sender As Object, e As EventArgs) Handles mnuSave.Click
		Using SaveDialog As New SaveFileDialog
			With SaveDialog
				.AddExtension = True
				.AutoUpgradeEnabled = True
				.CheckFileExists = False
				.CheckPathExists = True
				.CreatePrompt = False
				.DefaultExt = "rcj"
				.FileName = "MyRecipe.rcj"
				.Filter = "RecipeJet|*.rcj"
				.FilterIndex = 1
				.OverwritePrompt = True
				.ShowHelp = False
			End With

			If SaveDialog.ShowDialog = DialogResult.OK Then
				Using FileWriter As New StreamWriter(SaveDialog.FileName)
					FileWriter.WriteLine(txtTitle.Text)
					FileWriter.WriteLine(spnProcessTime.Value.ToString)
					FileWriter.WriteLine(spnTemperature.Value.ToString)
					FileWriter.WriteLine(spnN2Flow.Value.ToString)
				End Using
			End If
		End Using
	End Sub

	Private Sub mnuTopMost_Click(sender As Object, e As EventArgs) Handles mnuTopMost.Click
		TopMost = Not TopMost
	End Sub

	Private Sub mnuExit_Click(sender As Object, e As EventArgs) Handles mnuExit.Click
		Close()
	End Sub

	Private Sub mnuComm_DropDownOpening(sender As Object, e As EventArgs) Handles mnuComm.DropDownOpening
		mnuConnect.Enabled = (_QCPU Is Nothing)
		mnuDisconnect.Enabled = (_QCPU IsNot Nothing)
	End Sub

	Private Sub mnuConnect_Click(sender As Object, e As EventArgs) Handles mnuConnect.Click
		If _QCPU IsNot Nothing Then
			Exit Sub
		End If

		Using ConnectSetting As New frmConnect With {.Address = _CPUAddress, .Port = _ConnectionPort, .Encoding = DirectCast(_TextEncoding.CodePage, TextEncodeFlags), .ConnectAsync = True}
			If ConnectSetting.ShowDialog(Me) = DialogResult.OK Then
				_CPUAddress = ConnectSetting.Address
				_ConnectionPort = ConnectSetting.Port
				_TextEncoding = System.Text.Encoding.GetEncoding(ConnectSetting.Encoding)

				_QCPU = New MelsecCPU(CPUTypeFlags.QSeries)

				If ConnectSetting.ConnectAsync Then
					'接続処理を非同期で実施
					AddHandler _QCPU.ConnectionCreated, AddressOf QCPU_ConnectionCreated
					AddHandler _QCPU.ConnectionFailed, AddressOf QCPU_ConnectionFailed
					_QCPU.ConnectAsync(_CPUAddress, _ConnectionPort)
				Else
					'接続処理を同期で実施
					Try
						_QCPU.Connect(_CPUAddress, _ConnectionPort)
						QCPU_ConnectionCreated(_QCPU, New ConnectionCreatedEventArgs(_QCPU.CommClient))
					Catch ex As Exception
						QCPU_ConnectionFailed(sender, e)
					End Try
				End If
			End If
		End Using
	End Sub

	Private Sub mnuDisconnect_Click(sender As Object, e As EventArgs) Handles mnuDisconnect.Click
		InitializeComponent2()
	End Sub

	Private Sub mnuPLC_DropDownOpening(sender As Object, e As EventArgs) Handles mnuPLC.DropDownOpening
		mnuDeviceOperation.Enabled = (_QCPU IsNot Nothing)
	End Sub

	Private Sub mnuDeviceOperation_Click(sender As Object, e As EventArgs) Handles mnuDeviceOperation.Click
		Using DeviceOperationDialog As New frmDeviceOperation(_QCPU)
			DeviceOperationDialog.ShowDialog(Me)
		End Using
	End Sub


	Private Sub spnRecipeIndex_ValueChanged(sender As Object, e As EventArgs) 'Handles spnRecipeIndex.ValueChanged
		cmdLoad_Click(sender, e)
	End Sub


	'最短距離サンプル --- 8> ----- - ----- 8> ----- - ----- 8> ----- - ----- 8> ----- - ----- 8> ---
	Private Sub mnuSample_DropDownOpening(sender As Object, e As EventArgs) Handles mnuSample.DropDownOpening
		If DebugMonitor Is Nothing Then
			DebugMonitor = New frmDebugMonitor
			DebugMonitor.Show(Me)
		Else
			If Not DebugMonitor.Visible Then
				DebugMonitor.Location = New Point(Location.X + Width, Location.Y)
				DebugMonitor.Visible = True
			End If
		End If

		mnuWordWriteSample1.Enabled = (_QCPU IsNot Nothing)
		mnuWordWriteSample2.Enabled = (_QCPU IsNot Nothing)
		mnuWordWriteSample3.Enabled = (_QCPU IsNot Nothing)
		mnuWordReadSample1.Enabled = (_QCPU IsNot Nothing)
		mnuWordReadSample2.Enabled = (_QCPU IsNot Nothing)
		mnuWordReadSample3.Enabled = (_QCPU IsNot Nothing)
		mnuBitWriteSample1.Enabled = (_QCPU IsNot Nothing)
		mnuBitWriteSample2.Enabled = (_QCPU IsNot Nothing)
		mnuBitWriteSample3.Enabled = (_QCPU IsNot Nothing)
		mnuBitReadSample1.Enabled = (_QCPU IsNot Nothing)
		mnuBitReadSample2.Enabled = (_QCPU IsNot Nothing)
		mnuWordRandomWriteSample.Enabled = (_QCPU IsNot Nothing)
		mnuWordRandomReadSample.Enabled = (_QCPU IsNot Nothing)
		mnuBitRandomWriteSample.Enabled = (_QCPU IsNot Nothing)
		mnuFillSampleWordDeives0.Enabled = (_QCPU IsNot Nothing)
		mnuFillSampleWordDevices9999.Enabled = (_QCPU IsNot Nothing)
		mnuSetSampleBitDeivesFalse.Enabled = (_QCPU IsNot Nothing)
		mnuSetSampleBitDevicesTrue.Enabled = (_QCPU IsNot Nothing)
	End Sub

	Private Sub mnuFillSampleWordDeives0_Click(sender As Object, e As EventArgs) Handles mnuFillSampleWordDeives0.Click
		ClearSampleWordArea(0)
	End Sub

	Private Sub mnuFillSampleWordDevices9999_Click(sender As Object, e As EventArgs) Handles mnuFillSampleWordDevices9999.Click
		ClearSampleWordArea(9999)
	End Sub

	Private Sub mnuSetSampleBitDeivesFalse_Click(sender As Object, e As EventArgs) Handles mnuSetSampleBitDeivesFalse.Click
		ClearSampleBitArea(False)
	End Sub

	Private Sub mnuSetSampleBitDevicesTrue_Click(sender As Object, e As EventArgs) Handles mnuSetSampleBitDevicesTrue.Click
		ClearSampleBitArea(True)
	End Sub

	Private Sub ClearSampleWordArea(FillValue As Short)
		Dim DataBuilder As New DataBuilder

		For Offset As Integer = 0 To 29
			DataBuilder.SetInt16(FillValue)
		Next

		Using WordWriter As New WordWriter(_QCPU)
			WordWriter.Write(New WordAddress("D1000"), DataBuilder)
		End Using

		DebugMonitor.SetText("Fill sample word device " & FillValue.ToString & " done.")
		DebugMonitor.SetText("")
	End Sub

	Private Sub ClearSampleBitArea(FillValue As Boolean)
		Dim BitStatus As New List(Of Boolean)

		For Offset As Integer = 0 To 31
			BitStatus.Add(FillValue)
		Next

		Using BitWriter As New BitWriter(_QCPU)
			BitWriter.Write(New BitAddress("M1000"), BitStatus.ToArray)
			BitWriter.Write(New BitAddress("Y0"), BitStatus.ToArray)
		End Using

		DebugMonitor.SetText("Set sample bit device " & FillValue.ToString & " done.")
		DebugMonitor.SetText("")
	End Sub

	'ワード書き込み
	Private Sub Sample_WriteWords1(sender As Object, e As EventArgs) Handles mnuWordWriteSample1.Click
		'D レジスタにデータを書き込みます
		'・2 番目以降のデータは直前のデータに連続するため、書き込むアドレスを指定することはできません
		'・D1000 を先頭に下記データを書き込みます
		'	・1234 (16bit 符号無し整数 (1 Word)) > D1000
		'	・2345 (16bit 符号付き整数 (1 Word)) > D1001
		'	・3456 (32bit 符号無し整数 (2 Word)) > D1002, D1003
		'	・4567 (32bit 符号付き整数 (2 Word)) > D1004, D1005
		'	・"MELSEC" (文字列 (4 Word 占有)) > D1006 ～ D1009 ※D1009 は終端データ (&H0) が書き込まれます
		'	・"SYSMAC" (文字列 (3 Word 占有)) > D1010 ～ D1012 ※D1012 は終端データ (&H0) が書き込まれるため、引数に与えた文字列の 3 Word 目以降 ("AC") は破棄されます

		Dim StartAddress As New WordAddress(1000, WordDeviceFlags.D)
		Dim DataBuilder As New DataBuilder

		With DataBuilder
			.SetUInt16(1234)
			.SetInt16(2345)
			.SetUInt32(3456)
			.SetInt32(4567)

			If True Then
				.SetString("MELSEC", 4, PLCEncoding.ShiftJIS)
				.SetString("SYSMAC", 3, PLCEncoding.ShiftJIS)
			Else
				'SetString メソッドのオーバーロードは、使用する前に文字列のデフォルトエンコードを設定してください
				.DefaultEncoding = PLCEncoding.ShiftJIS
				.SetString("MELSEC", 4)
				.SetString("SYSMAC", 3)
			End If
		End With

		Using WordWriter As New WordWriter(_QCPU)
			WordWriter.Write(StartAddress, DataBuilder)
		End Using

		DebugMonitor.SetText("Write word data (1) done.")
		DebugMonitor.SetText("")
	End Sub

	Private Sub Sample_WriteWords2(sender As Object, e As EventArgs) Handles mnuWordWriteSample2.Click
		'Sub Sample_WriteWords1() の内容を別の方法で書いてみます
		'・書き込みアドレスを文字列で渡す
		'・書き込みデータを Byte 配列で渡す (ここでは可用性を示すため、意図的に SetAnyBytes と Set[Type] を混在させています)

		Dim StartAddress As New WordAddress("D1000")
		Dim DataBuilder As New DataBuilder

		With DataBuilder
			.SetAnyBytes(DeviceConverter.UInt16ToByteArray(1234))
			.SetInt16(2345)
			.SetAnyBytes(DeviceConverter.UInt32ToByteArray(3456))
			.SetInt32(4567)
			.SetAnyBytes(DeviceConverter.StringToByteArray("MELSEC", 4, PLCEncoding.ShiftJIS))
			.SetString("SYSMAC", 3, PLCEncoding.ShiftJIS)
		End With

		Using WordWriter As New WordWriter(_QCPU)
			WordWriter.Write(StartAddress, DataBuilder)
		End Using

		DebugMonitor.SetText("Write word data (2) done.")
		DebugMonitor.SetText("")
	End Sub

	Private Sub Sample_WriteWords3(sender As Object, e As EventArgs) Handles mnuWordWriteSample3.Click
		'Sub Sample_WriteWords1() の内容を別の方法で書いてみます
		'・WordWriter クラスの QuickWrite 共有メソッドを使用する

		WordWriter.QuickWrite(_QCPU, New WordAddress("D1000"), 1234US)
		WordWriter.QuickWrite(_QCPU, New WordAddress("D1001"), 2345S)
		WordWriter.QuickWrite(_QCPU, New WordAddress("D1002"), 3456UI)
		WordWriter.QuickWrite(_QCPU, New WordAddress("D1004"), 4567I)
		WordWriter.QuickWrite(_QCPU, New WordAddress("D1006"), "MELSEC", 4, PLCEncoding.ShiftJIS)
		WordWriter.QuickWrite(_QCPU, New WordAddress("D1010"), "SYSMAC", 3, PLCEncoding.ShiftJIS)

		DebugMonitor.SetText("Write word data (3) done.")
		DebugMonitor.SetText("")
	End Sub

	'ワード読み込み
	Private Sub Sample_ReadWords1(sender As Object, e As EventArgs) Handles mnuWordReadSample1.Click
		'D レジスタのデータを読み出します
		'・D1000 を先頭に下記構造になっているものとして取り扱います
		'	・D1000: 16bit 符号無し整数 (1 Word)
		'	・D1001: 16bit 符号付き整数 (1 Word)
		'	・D1002: 32bit 符号無し整数 (2 Word)
		'	・D1004: 32bit 符号付き整数 (2 Word)
		'	・D1006: 文字列 (4 Word) ※終端データ含む
		'	・D1010: 文字列 (3 Word) ※終端データ含む

		Dim StartAddress As New WordAddress(1000, WordDeviceFlags.D)
		Dim Response As Byte()
		Dim DataBuilder As DataBuilder

		'1) D1000 を先頭に 13 Word 読み出し、DataBuilder のコンストラクタへ与えます
		Using WordReader As New WordReader(_QCPU)
			Response = WordReader.Read(StartAddress, 13)
			DataBuilder = New DataBuilder(Response)
		End Using

		'2) 読み出したデータを DataBuilder を通じて型変換・取得します
		Dim UInt16Response As UInt16
		Dim Int16Response As Int16
		Dim UInt32Response As UInt32
		Dim Int32Response As Int32
		Dim StringResponse1 As String
		Dim StringResponse2 As String

		'<重要> Get*** メソッドを実行する順序が、PLC のアドレス順に一致するようにしてください
		UInt16Response = DataBuilder.GetUInt16
		Int16Response = DataBuilder.GetInt16
		UInt32Response = DataBuilder.GetUInt32
		Int32Response = DataBuilder.GetInt32

		If True Then
			StringResponse1 = DataBuilder.GetString(4, PLCEncoding.ShiftJIS)
			StringResponse2 = DataBuilder.GetString(3, PLCEncoding.ShiftJIS)
		Else
			'SetString メソッドのオーバーロードは、使用する前に文字列のデフォルトエンコードを設定してください
			DataBuilder.DefaultEncoding = PLCEncoding.ShiftJIS
			StringResponse1 = DataBuilder.GetString(4)
			StringResponse2 = DataBuilder.GetString(3)
		End If

		DebugMonitor.SetText(String.Format("D1000 = {0}", UInt16Response))
		DebugMonitor.SetText(String.Format("D1001 = {0}", Int16Response))
		DebugMonitor.SetText(String.Format("D1002 = {0}", UInt32Response))
		DebugMonitor.SetText(String.Format("D1004 = {0}", Int32Response))
		DebugMonitor.SetText(String.Format("D1006 = '{0}'", StringResponse1))
		DebugMonitor.SetText(String.Format("D1010 = '{0}'", StringResponse2))
		DebugMonitor.SetText("Read word data (1) done.")
		DebugMonitor.SetText("")
	End Sub

	Private Sub Sample_ReadWord2(sender As Object, e As EventArgs) Handles mnuWordReadSample2.Click
		'Sub Sample_ReadWords1() の内容を別の方法で書いてみます
		'・読み取りデータを Byte 配列で受け取り、自身で型変換を行う

		Dim StartAddress As New WordAddress(1000, WordDeviceFlags.D)
		Dim Response As List(Of Byte)

		'1) D1000 を先頭に 13 Word (=26 bytes) 読み出します
		Using WordReader As New WordReader(_QCPU)
			Response = WordReader.Read(StartAddress, 13).ToList
		End Using

		'2) 読み出したデータを DeviceConverter クラスの共有メソッドで型変換・取得します
		Dim UInt16Response As UInt16
		Dim Int16Response As Int16
		Dim UInt32Response As UInt32
		Dim Int32Response As Int32
		Dim StringResponse1a As String
		Dim StringResponse2a As String
		Dim StringResponse1b As String
		Dim StringResponse2b As String

		UInt16Response = DeviceConverter.ByteToUInt16({Response.Item(0), Response.Item(1)})
		Int16Response = DeviceConverter.ByteToInt16({Response.Item(2), Response.Item(3)})
		UInt32Response = DeviceConverter.ByteToUInt16(Response.ToList.GetRange(4, 4).ToArray)
		Int32Response = DeviceConverter.ByteToInt32(Response.ToList.GetRange(8, 4).ToArray)
		StringResponse1a = DeviceConverter.ByteToString(Response.GetRange(12, 8).ToArray, PLCEncoding.ShiftJIS)
		StringResponse2a = DeviceConverter.ByteToString(Response.GetRange(20, 6).ToArray, PLCEncoding.ShiftJIS)

		'なお、ByteToString に渡す Byte 配列に終端データが含まれていなくても変換可能です
		StringResponse1b = DeviceConverter.ByteToString(Response.GetRange(12, 6).ToArray, PLCEncoding.ShiftJIS)
		StringResponse2b = DeviceConverter.ByteToString(Response.GetRange(20, 4).ToArray, PLCEncoding.ShiftJIS)

		DebugMonitor.SetText(String.Format("D1000 = {0}", UInt16Response))
		DebugMonitor.SetText(String.Format("D1001 = {0}", Int16Response))
		DebugMonitor.SetText(String.Format("D1002 = {0}", UInt32Response))
		DebugMonitor.SetText(String.Format("D1004 = {0}", Int32Response))
		DebugMonitor.SetText(String.Format("D1006 (その1) = '{0}'", StringResponse1a))
		DebugMonitor.SetText(String.Format("D1010 (その1) = '{0}'", StringResponse2a))
		DebugMonitor.SetText(String.Format("D1006 (その2) = '{0}'", StringResponse1b))
		DebugMonitor.SetText(String.Format("D1010 (その2) = '{0}'", StringResponse2b))
		DebugMonitor.SetText("Read word data (2) done.")
		DebugMonitor.SetText("")
	End Sub

	Private Sub Sample_ReadWord3(sender As Object, e As EventArgs) Handles mnuWordReadSample3.Click
		'Sub Sample_ReadWords1() の内容を別の方法で書いてみます
		'・WordReader クラスの QuickRead 共有メソッドを使用する

		Dim UInt16Response As UInt16
		Dim Int16Response As Int16
		Dim UInt32Response As UInt32
		Dim Int32Response As Int32
		Dim StringResponse1 As String
		Dim StringResponse2 As String

		UInt16Response = CUShort(WordReader.QuickRead(_QCPU, New WordAddress("D1000"), NumericDataTypeFlags.UnsignedSingleInt))
		Int16Response = CShort(WordReader.QuickRead(_QCPU, New WordAddress("D1001"), NumericDataTypeFlags.SignedSingleInt))
		UInt32Response = CUInt(WordReader.QuickRead(_QCPU, New WordAddress("D1002"), NumericDataTypeFlags.UnsignedDoubleInt))
		Int32Response = CInt(WordReader.QuickRead(_QCPU, New WordAddress("D1004"), NumericDataTypeFlags.SignedSingleInt))
		StringResponse1 = WordReader.QuickRead(_QCPU, New WordAddress("D1006"), 4, PLCEncoding.ShiftJIS)
		StringResponse2 = WordReader.QuickRead(_QCPU, New WordAddress("D1010"), 3, PLCEncoding.ShiftJIS)

		DebugMonitor.SetText(String.Format("D1000 = {0}", UInt16Response))
		DebugMonitor.SetText(String.Format("D1001 = {0}", Int16Response))
		DebugMonitor.SetText(String.Format("D1002 = {0}", UInt32Response))
		DebugMonitor.SetText(String.Format("D1004 = {0}", Int32Response))
		DebugMonitor.SetText(String.Format("D1006 = '{0}'", StringResponse1))
		DebugMonitor.SetText(String.Format("D1010 = '{0}'", StringResponse2))
		DebugMonitor.SetText("Read word data (3) done.")
		DebugMonitor.SetText("")
	End Sub

	'ビット書き込み
	Private Sub Sample_WriteBits1(sender As Object, e As EventArgs) Handles mnuBitWriteSample1.Click
		'M リレーにデータを書き込みます
		'M1000 を先頭に下記状態を書き込みます
		'	・True > M1000
		'	・False > M1001
		'	・True > M1002
		'	・True > M1003
		'	・False > M1004
		'	・True > M1005
		'	・False > M1006
		'	・True > M1007

		Dim StartAddress As New BitAddress(1000, BitDeviceFlags.M)
		Dim WriteData As New List(Of Boolean)

		With WriteData
			.Add(True) 'M1000
			.Add(False) 'M1001
			.Add(True) 'M1002
			.Add(True) 'M1003
			.Add(False) 'M1004
			.Add(True) 'M1005
			.Add(False) 'M1006
			.Add(True) 'M1007
		End With

		Using BitWriter As New BitWriter(_QCPU)
			BitWriter.Write(StartAddress, WriteData.ToArray)
		End Using

		DebugMonitor.SetText("Write bit data (1) done.")
		DebugMonitor.SetText("")
	End Sub

	Private Sub Sample_WriteBits2(sender As Object, e As EventArgs) Handles mnuBitWriteSample2.Click
		'Sub Sample_WriteBits1() の内容を別の方法で書いてみます

		Dim StartAddress As New BitAddress("M1000")
		Dim WriteData As Boolean() = {True, False, True, True, False, True, False, True}

		Using BitWriter As New BitWriter(_QCPU)
			BitWriter.Write(StartAddress, WriteData)
		End Using

		DebugMonitor.SetText("Write bit data (2) done.")
		DebugMonitor.SetText("")
	End Sub

	Private Sub Sample_WriteBits3(sender As Object, e As EventArgs) Handles mnuBitWriteSample3.Click
		'Sub Sample_WriteBits1() の内容を別の方法で書いてみます
		'・BitWriter クラスの QuickWrite 共有メソッドを使用する

		BitWriter.QuickWrite(_QCPU, New BitAddress("M1000"), True)
		BitWriter.QuickWrite(_QCPU, New BitAddress("M1001"), False)
		BitWriter.QuickWrite(_QCPU, New BitAddress("M1002"), True)
		BitWriter.QuickWrite(_QCPU, New BitAddress("M1003"), True)
		BitWriter.QuickWrite(_QCPU, New BitAddress("M1004"), False)
		BitWriter.QuickWrite(_QCPU, New BitAddress("M1005"), True)
		BitWriter.QuickWrite(_QCPU, New BitAddress("M1006"), False)
		BitWriter.QuickWrite(_QCPU, New BitAddress("M1007"), True)

		DebugMonitor.SetText("Write bit data (3) done.")
		DebugMonitor.SetText("")
	End Sub

	'ビット読み込み
	Private Sub Sample_ReadBits1(sender As Object, e As EventArgs) Handles mnuBitReadSample1.Click
		'M リレーの状態を読み出します
		'・M1000 を先頭に 8 bit 読み出します

		Dim StartAddress As New BitAddress(1000, BitDeviceFlags.M)
		Dim Response As List(Of Boolean)

		Using BitReader As New BitReader(_QCPU)
			Response = BitReader.Read(StartAddress, 8)
		End Using

		Dim M1000 As Boolean = Response.Item(0)
		Dim M1001 As Boolean = Response.Item(1)
		Dim M1002 As Boolean = Response.Item(2)
		Dim M1003 As Boolean = Response.Item(3)
		Dim M1004 As Boolean = Response.Item(4)
		Dim M1005 As Boolean = Response.Item(5)
		Dim M1006 As Boolean = Response.Item(6)
		Dim M1007 As Boolean = Response.Item(7)

		DebugMonitor.SetText(String.Format("M1000 = {0}", M1000))
		DebugMonitor.SetText(String.Format("M1001 = {0}", M1001))
		DebugMonitor.SetText(String.Format("M1002 = {0}", M1002))
		DebugMonitor.SetText(String.Format("M1003 = {0}", M1003))
		DebugMonitor.SetText(String.Format("M1004 = {0}", M1004))
		DebugMonitor.SetText(String.Format("M1005 = {0}", M1005))
		DebugMonitor.SetText(String.Format("M1006 = {0}", M1006))
		DebugMonitor.SetText(String.Format("M1007 = {0}", M1007))
		DebugMonitor.SetText("Read bit data (1) done.")
		DebugMonitor.SetText("")
	End Sub

	Private Sub Sample_ReadBits2(sender As Object, e As EventArgs) Handles mnuBitReadSample2.Click
		'Sub Sample_ReadBits1() の内容を別の方法で書いてみます
		'・BitReader クラスの QuickRead 共有メソッドを使用する

		Dim M1000 As Boolean = BitReader.QuickRead(_QCPU, New BitAddress("M1000"))
		Dim M1001 As Boolean = BitReader.QuickRead(_QCPU, New BitAddress("M1001"))
		Dim M1002 As Boolean = BitReader.QuickRead(_QCPU, New BitAddress("M1002"))
		Dim M1003 As Boolean = BitReader.QuickRead(_QCPU, New BitAddress("M1003"))
		Dim M1004 As Boolean = BitReader.QuickRead(_QCPU, New BitAddress("M1004"))
		Dim M1005 As Boolean = BitReader.QuickRead(_QCPU, New BitAddress("M1005"))
		Dim M1006 As Boolean = BitReader.QuickRead(_QCPU, New BitAddress("M1006"))
		Dim M1007 As Boolean = BitReader.QuickRead(_QCPU, New BitAddress("M1007"))

		DebugMonitor.SetText(String.Format("M1000 = {0}", M1000))
		DebugMonitor.SetText(String.Format("M1001 = {0}", M1001))
		DebugMonitor.SetText(String.Format("M1002 = {0}", M1002))
		DebugMonitor.SetText(String.Format("M1003 = {0}", M1003))
		DebugMonitor.SetText(String.Format("M1004 = {0}", M1004))
		DebugMonitor.SetText(String.Format("M1005 = {0}", M1005))
		DebugMonitor.SetText(String.Format("M1006 = {0}", M1006))
		DebugMonitor.SetText(String.Format("M1007 = {0}", M1007))
		DebugMonitor.SetText("Read bit data (2) done.")
		DebugMonitor.SetText("")
	End Sub

	'ランダムアクセス
	Private Sub Sample_RandomWordWrite(sender As Object, e As EventArgs) Handles mnuWordRandomWriteSample.Click
		'D レジスタの任意の位置にデータを書き込みます
		'・書き込みデータは WordDeviceCollection オブジェクトを利用して構築します
		'・下記データを書き込みます
		'	・D1000: 1234 (16bit 符号無し整数 (1 Word))
		'	・D1002: 2345 (16bit 符号付き整数 (1 Word))
		'	・D1004: 3456 (32bit 符号無し整数 (2 Word))
		'	・D1007: 4567 (32bit 符号付き整数 (2 Word))
		'	※D1001, D1003, D1006, D1009 の値は影響されません

		Dim RandomWords As New WordDeviceCollection

		With RandomWords
			.Add(New UInt16Device(New WordAddress(1000, WordDeviceFlags.D), 1234US))
			.Add(New Int16Device(New WordAddress(1002, WordDeviceFlags.D), 2345S))
			.Add(New UInt32Device(New WordAddress(1004, WordDeviceFlags.D), 3456UI))
			.Add(New Int32Device(New WordAddress(1007, WordDeviceFlags.D), 4567I))
		End With

		Using RandomWordWriter As New RandomWordWriter(_QCPU)
			RandomWordWriter.Write(RandomWords)
		End Using

		DebugMonitor.SetText("Write random word data done.")
		DebugMonitor.SetText("")
	End Sub

	Private Sub Sample_RandomWordRead(sender As Object, e As EventArgs) Handles mnuWordRandomReadSample.Click
		'D レジスタの任意の位置のデータを読み出します
		'・読み出すデータは WordDeviceCollection オブジェクトを通じて指定します
		'・読み出されたデータは WordDeviceCollection を通じて取得します
		'・下記データを読み出します
		'	・D1000: 16bit 符号無し整数 (1 Word)
		'	・D1002: 16bit 符号付き整数 (1 Word)
		'	・D1004: 32bit 符号無し整数 (2 Word)
		'	・D1007: 32bit 符号付き整数 (2 Word)

		Dim RandomWords As New WordDeviceCollection

		'1) 読み出すデータを定義します
		With RandomWords
			.Add(New UInt16Device(New WordAddress("D1000"), 0))
			.Add(New Int16Device(New WordAddress("D1002"), 0))
			.Add(New UInt32Device(New WordAddress("D1004"), 0))
			.Add(New Int32Device(New WordAddress("D1007"), 0))
		End With

		Dim Response As Dictionary(Of String, Object)
		Dim D1000 As UInt16
		Dim D1002 As Int16
		Dim D1004 As UInt32
		Dim D1007 As Int32

		'2) データを読み出します
		Using RandomWordReader As New RandomWordReader(_QCPU)
			Response = RandomWordReader.Read(RandomWords)
		End Using

		D1000 = CUShort(Response.Item("D1000"))
		D1002 = CShort(Response.Item("D1002"))
		D1004 = CUInt(Response.Item("D1004"))
		D1007 = CInt(Response.Item("D1007"))

		DebugMonitor.SetText(String.Format("{0} = {1}", "D1000", D1000))
		DebugMonitor.SetText(String.Format("{0} = {1}", "D1002", D1002))
		DebugMonitor.SetText(String.Format("{0} = {1}", "D1004", D1004))
		DebugMonitor.SetText(String.Format("{0} = {1}", "D1007", D1007))
		DebugMonitor.SetText("Read random word data done.")
		DebugMonitor.SetText("")
	End Sub

	Private Sub Sample_RandomBitWrite(sender As Object, e As EventArgs) Handles mnuBitRandomWriteSample.Click
		'M リレー、Yリレー の任意の位置の状態を変更します
		'・書き込みデータは BitDeviceCollection オブジェクトを利用して構築します
		'下記状態を書き込みます
		'	・M1000: True
		'	・M1003: False
		'	・Y3: True
		'	・Y1A: False
		'	・Y1C: True
		'	・Y1D: False

		Dim RandomBits As New BitDeviceCollection

		With RandomBits
			.Add(New BitDevice(New BitAddress("M1000"), True))
			.Add(New BitDevice(New BitAddress(1003, BitDeviceFlags.M), False))
			.Add(New BitDevice(New BitAddress("Y3"), True))
			.Add(New BitDevice(New BitAddress(26, BitDeviceFlags.Y), False))
			.Add(New BitDevice(New BitAddress(&H1C, BitDeviceFlags.Y), True))
			.Add(New BitDevice(New BitAddress(1, &HD, BitDeviceFlags.Y), False))
		End With

		Using RandomBitWriter As New RandomBitWriter(_QCPU)
			RandomBitWriter.Write(RandomBits)
		End Using

		DebugMonitor.SetText("Write random bit data done.")
		DebugMonitor.SetText("")
	End Sub

	Private Sub Sample_RandomBitRead(sender As Object, e As EventArgs) Handles mnuBitRandomReadSample.Click
		'M リレー、X リレー、Y リレー の任意の位置の状態を取得します
		'読み取りデータは BitDeviceCollection オブジェクトを利用して構築します
		'下記ビットデバイスの状態を読み取ります
		'・M: M100, M102, M103
		'・X: X5, X9, XA
		'・Y: Y12, Y1C, Y1D

		Dim RandomBits As New BitDeviceCollection

		'1) 読み出すデータを定義します
		With RandomBits
			.Add(New BitDevice(New BitAddress("M100"), False))
			.Add(New BitDevice(New BitAddress("M102"), False))
			.Add(New BitDevice(New BitAddress("M103"), False))
			.Add(New BitDevice(New BitAddress("X5"), False))
			.Add(New BitDevice(New BitAddress("X9"), False))
			.Add(New BitDevice(New BitAddress("XA"), False))
			.Add(New BitDevice(New BitAddress("Y12"), False))
			.Add(New BitDevice(New BitAddress("Y1C"), False))
			.Add(New BitDevice(New BitAddress("Y1D"), False))
		End With

		Dim Response As Dictionary(Of String, Boolean)

		'2) データを読み出します
		Using RandomBitReader As New RandomBitReader(_QCPU)
			Response = RandomBitReader.Read(RandomBits)
		End Using

		Dim M100, M102, M103 As Boolean
		Dim X5, X9, XA As Boolean
		Dim Y12, Y1C, Y1D As Boolean

		M100 = Response.Item("M100")
		M102 = Response.Item("M102")
		M103 = Response.Item("M103")

		X5 = Response.Item("X5")
		X9 = Response.Item("X9")
		XA = Response.Item("XA")

		Y12 = Response.Item("Y12")
		Y1C = Response.Item("Y1C")
		Y1D = Response.Item("Y1D")

		For Each Status As KeyValuePair(Of String, Boolean) In Response
			DebugMonitor.SetText(Status.Key & " = " & Status.Value.ToString)
		Next
		DebugMonitor.SetText("Read random bit data done.")
		DebugMonitor.SetText("")
	End Sub
End Class